#include "os.h"
#include "s_defines.h"
#include "match_ext.h"
#include "s_externs.h"

#ifdef RUSNET_IRCD
#include "rusnet_cmds_ext.h"

int m_codepage(cptr, sptr, parc, parv)
aClient *cptr;
aClient *sptr;
int     parc;
char    *parv[];
{
    int  i;

    if (parc < 2) return -1;

    for (i = 0; parv[1][i] != '\0'; i++) parv[1][i] = toupper(parv[1][i]); 
 
    rusnet_changecodepage(sptr, parv[1], parv[0]);

    return 0;    
}

int m_force(cptr, sptr, parc, parv)
aClient *cptr, *sptr;
int     parc;
char    *parv[];
{
    aClient *acptr, *workptr, *zptr;

    if (parc < 3)
	return -1;

    /* Check if parv[2] is a registered nickname */
    workptr = NULL;
    if (find_client( parv[2], workptr) != NULL)
	return -1; /* Could not force nickchange, new nick is registered */

    /* Check if parv[1] exists and local */

    workptr = NULL;
    if ((zptr = find_client( parv[1], workptr)) == NULL)
       return -1; /* No such user */

    if (MyConnect(zptr))
    {  
	char    *pparv[3];

        /* Do nick change procedure */
        pparv[0] = parv[1];
        pparv[1] = parv[2];
        pparv[2] = NULL;
        m_nick(zptr, zptr, 2, pparv);
    }
    else
    {
      /* Propagate FORCE to single server where $nick is registered */
      acptr = zptr->from;
      if ((acptr != cptr) &&                     /* Split horizon */
	  (acptr->serv->version & SV_FORCE))	/* don't send to old servers */
	  sendto_one(acptr,"FORCE %s %s", parv[1], parv[2]);
    }

    return 0;
}

int m_rmode(cptr, sptr, parc, parv)
aClient *cptr, *sptr;
int     parc;
char    *parv[];
{
    aClient *acptr, *workptr, *zptr;
    char    *s;

    if (parc < 3)
	return -1;
#if 0
    /* cut more than 4 parrameters */
    
    parv[2][4]='\0';
#endif
    /* restricted O line */

    if ( *parv[2] == '+' ) 
	for (s = (char *) parv[2] + 1; *s != '\0'; s++) 
		if (*s == 'o' || *s == 'O') 
			return -1 ;

    /* Check if parv[1] exists and local */

    workptr = NULL;

    if ((zptr = find_client(parv[1], workptr)) == NULL)
	return -1; /* No such user */

    if (MyConnect(zptr))
    {
	char    *pparv[4];

	if ( *parv[2] == '+' ) 
	    for (s = (char *) parv[2] + 1; *s != '\0'; s++) 
    		if (*s == 'h' && !IsOper(zptr)) 
    			return -1;	/* deny non-opered shadows */

	/* Do change mode procedure */

	pparv[0] = parv[1];
	pparv[1] = parv[1];
	pparv[2] = parv[2];
	pparv[3] = NULL;        

	/* sending to s_user.c */

	m_umode(NULL, zptr, 3, pparv); /* internal fake call */
    }
    else
    {
      acptr = zptr->from;
      if ((acptr != cptr) &&                     /* Split horizon */
	  (acptr->serv->version & SV_RMODE))   /* don't send to old servers */
	  sendto_one(acptr,"RMODE %s %s", parv[1], parv[2]);
    }

    return 0;
}

int m_rcpage(cptr, sptr, parc, parv)
aClient *cptr, *sptr;
int     parc;
char    *parv[];
{
    aClient *acptr, *workptr, *zptr;

    if (parc < 3)
	return -1;

    /* Check if parv[1] exists and local */

    workptr = NULL;

    if ((zptr = find_client(parv[1], workptr)) == NULL)
       return -1; /* No such user */

    if (MyConnect(zptr))
    { 
	int i;

        /* Do change codepage procedure */
	for (i = 0; parv[2][i] != '\0'; i++) parv[2][i] = toupper(parv[2][i]); 

        rusnet_changecodepage(zptr, parv[2], parv[1]);
    }
    else
    {
      acptr = zptr->from;
      if ((acptr != cptr) &&                     /* Split horizon */
	  (acptr->serv->version & SV_RMODE))   /* don't send to old servers */
	  sendto_one(acptr,"RCPAGE %s %s", parv[1], parv[2]);
    }

    return 0;
}

int m_kline(cptr, sptr, parc, parv)
aClient *cptr, *sptr;
int     parc;
char    *parv[];
{
    aClient *acptr, *workptr, *zptr;

    /*
     * KLINE server nickmask usermask hostmask [hours] 
     * Wildcards in masks are allowed. No hours means forever
     */

    if (parc < 5)
	return -1;		/* not enough parameters */

    /* Check if parv[2] exists and local */

    workptr = NULL;

    if ((zptr = find_server(parv[1], workptr)) == NULL)
       return -1;		/* No such server known */

    if (IsMe(zptr))
    {  

	if ((acptr = best_service("TkServ", workptr))== NULL)
		return -1;
	
	if (!MyConnect(acptr))
		return -1;
		
	/*
	 * Call for TkServ:
	 * TKLINE <password> <lifetime> <user@host> <reason>
	 * Nick masks should be mapped to user masks if present
	 */

	sendto_one(acptr, ":%s SQUERY TkServ :TKLINE %s %s %s@%s %s",
			TK_USER, TK_PASS, parv[5] ? parv[5] : TK_MAXTIME,
		/* we use <user> if it is NOT equal to "*", be attentional */
			(strcmp(parv[3], "*")) ? parv[3] : parv[2],
							parv[4], TK_REASON);
    }
    else
    {
      acptr = zptr->from;
      if ((acptr != cptr) &&                     /* Split horizon */
	  (acptr->serv->version & SV_KLINE))   /* don't send to old servers */
	  sendto_one(acptr, "KLINE %s %s %s %s %s", parv[1], parv[2],
			parv[3], parv[4], parv[5] ? parv[5] : "");
    }

    return 0;
}

#endif
